@@ -1,12 +1,10 @@
 /*
- * $Id: dns.h,v 1.14 2003/07/07 09:42:21 erik Exp $
- *
  * dns.h -- everything we wanted to know but were afraid
  *		to ask about DNS
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2004, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -45,6 +43,7 @@
 /* RFC1035 */
 #define	CLASS_IN	1	/* Class IN */
 #define	CLASS_CHAOS	3	/* Class CHAOS */
+#define CLASS_HS        4       /* Class HS */
 #define	CLASS_ANY	255	/* Class IN */
 
 #define TYPE_A		1	/* a host address */
@@ -65,21 +64,42 @@
 #define TYPE_TXT	16	/* text strings */
 #define	TYPE_RP		17	/* RFC1183 */
 #define	TYPE_AFSDB	18	/* RFC1183 */
-#define	TYPE_SIG	24	/* RFC2065 */
-#define	TYPE_KEY	25	/* RFC2065 */
+
+#define	TYPE_SIG	24	/* 2535typecode */
+#define	TYPE_KEY	25	/* 2535typecode */
+#define	TYPE_NXT	30 	/* 2535typecode */
+
+/* type code rollover */
+#ifdef TCR_USE_OLD_TYPES /* old types */
+#define	TYPE_RRSIG	TYPE_SIG	/* map to old type codes */
+#define	TYPE_NSEC	TYPE_NXT 	
+#define	TYPE_DNSKEY	TYPE_KEY
+#else	/* new types */
+#define TYPE_RRSIG	46	/* draft-ietf-dnsext-dnssec-25 */
+#define TYPE_NSEC	47	
+#define TYPE_DNSKEY	48
+#endif
+
 #define TYPE_AAAA	28	/* ipv6 address */
 #define TYPE_LOC	29	/* LOC record  RFC1876 */
-#define	TYPE_NXT	30	/* RFC2065 */
 #define	TYPE_SRV	33	/* SRV record RFC2782 */
 #define	TYPE_NAPTR	35	/* NAPTR record RFC2915 */
 #define	TYPE_OPT	41	/* Pseudo OPT record... */
 #define	TYPE_DS		43	/* draft-ietf-dnsext-delegation */
+#define TYPE_SSHFP	44	/* SSH Key Fingerprint */
 #define	TYPE_IXFR	251
 #define	TYPE_AXFR	252
 #define	TYPE_MAILB	253 	/* A request for mailbox-related records (MB, MG or MR) */
 #define	TYPE_MAILA	254	/* A request for mail agent RRs (Obsolete - see MX) */
 #define TYPE_ANY	255	/* any type (wildcard) */
 
-#define MAXDOMAINLEN	255
+#define	MAXLABELLEN	63
+#define	MAXDOMAINLEN	255
+
+#define MAX_RDLENGTH            65535
+
+/* Maximum size of a single RR.  */
+#define MAX_RR_SIZE \
+	(MAXDOMAINLEN + sizeof(uint32_t) + 4*sizeof(uint16_t) + MAX_RDLENGTH)
 
 #endif /* _DNS_H_ */
