@@ -3,7 +3,7 @@
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2004, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -82,19 +82,20 @@ static void
 usage (void)
 {
 	fprintf(stderr, "Usage: nsd [OPTION]...\n");
-	fprintf(stderr, "Start the NSD name server daemon.\n\n");
+	fprintf(stderr, "Name Server Daemon.\n\n");
 	fprintf(stderr,
 		"Supported options:\n"
 		"  -4              Only listen to IPv4 connections.\n"
 		"  -6              Only listen to IPv6 connections.\n"
+		"  -A              Set the AD bit on answers from secure zones.\n"
 		"  -a ip-address   Listen to the specified incoming IP address (may be\n"
 		"                  specified multiple times).\n"
 		"  -d              Enable debug mode (do not fork as a daemon process).\n"
 		"  -f database     Specify the database to load.\n"
 		"  -h              Print this help information.\n"
-		"  -i identity     Specify the identity when queried for id.server CHAOS TXT.\n"
 		);
 	fprintf(stderr,
+		"  -i identity     Specify the identity when queried for id.server CHAOS TXT.\n"
 		"  -l filename     Specify the log file.\n"
 		"  -N udp-servers  Specify the number of child UDP servers.\n"
 		"  -n tcp-servers  Specify the number of child TCP servers.\n"
@@ -102,6 +103,8 @@ usage (void)
 		"  -s seconds      Dump statistics every SECONDS seconds.\n"
 		"  -t chrootdir    Change root to specified directory on startup.\n"
 		"  -u user         Change effective uid to the specified user.\n"
+		);
+	fprintf(stderr,
 		"  -v              Print version information.\n"
 		"  -X plugin       Load a plugin (may be specified multiple times).\n\n"
 		);
@@ -115,7 +118,7 @@ version(void)
 	fprintf(stderr, "%s version %s\n", PACKAGE_NAME, PACKAGE_VERSION);
 	fprintf(stderr, "Written by NLnet Labs.\n\n");
 	fprintf(stderr,
-		"Copyright (C) 2001-2003 NLnet Labs.  This is free software.\n"
+		"Copyright (C) 2001-2004 NLnet Labs.  This is free software.\n"
 		"There is NO warranty; not even for MERCHANTABILITY or FITNESS\n"
 		"FOR A PARTICULAR PURPOSE.\n");
 	exit(0);
@@ -140,11 +143,11 @@ readpid (const char *file)
 	char *t;
 	int l;
 
-	if((fd = open(file, O_RDONLY)) == -1) {
+	if ((fd = open(file, O_RDONLY)) == -1) {
 		return -1;
 	}
 
-	if(((l = read(fd, pidbuf, sizeof(pidbuf)))) == -1) {
+	if (((l = read(fd, pidbuf, sizeof(pidbuf)))) == -1) {
 		close(fd);
 		return -1;
 	}
@@ -152,14 +155,14 @@ readpid (const char *file)
 	close(fd);
 
 	/* Empty pidfile means no pidfile... */
-	if(l == 0) {
+	if (l == 0) {
 		errno = ENOENT;
 		return -1;
 	}
 
 	pid = strtol(pidbuf, &t, 10);
 
-	if(*t && *t != '\n') {
+	if (*t && *t != '\n') {
 		return -1;
 	}
 	return pid;
@@ -168,24 +171,25 @@ readpid (const char *file)
 int 
 writepid (struct nsd *nsd)
 {
-	int fd;
+	FILE * fd;
 	char pidbuf[16];
 
 	snprintf(pidbuf, sizeof(pidbuf), "%lu\n", (unsigned long) nsd->pid);
 
-	if((fd = open(nsd->pidfile, O_WRONLY | O_TRUNC | O_CREAT, 0644)) == -1) {
+	if ((fd = fopen(nsd->pidfile, "w")) ==  NULL ) {
 		return -1;
 	}
 
-	if((write(fd, pidbuf, strlen(pidbuf))) == -1) {
-		close(fd);
+	if (!write_data(fd, pidbuf, strlen(pidbuf))) {
+		fclose(fd);
 		return -1;
 	}
-	close(fd);
+	fclose(fd);
 
-	if(chown(nsd->pidfile, nsd->uid, nsd->gid) == -1) {
+	if (chown(nsd->pidfile, nsd->uid, nsd->gid) == -1) {
 		log_msg(LOG_ERR, "cannot chown %u.%u %s: %s",
-			nsd->uid, nsd->gid, nsd->pidfile, strerror(errno));
+			(unsigned) nsd->uid, (unsigned) nsd->gid,
+			nsd->pidfile, strerror(errno));
 		return -1;
 	}
 
@@ -200,7 +204,7 @@ sig_handler (int sig)
 	
 	/* Are we a child server? */
 	if (nsd.server_kind != NSD_SERVER_MAIN) {
-		switch(sig) {
+		switch (sig) {
 		case SIGCHLD:
 			/* Plugins may fork, reap all terminated children.  */
 			while (waitpid(0, NULL, WNOHANG) > 0)
@@ -222,7 +226,7 @@ sig_handler (int sig)
 		return;
 	}
 
-	switch(sig) {
+	switch (sig) {
 	case SIGCHLD:
 		return;
 	case SIGHUP:
@@ -251,7 +255,7 @@ sig_handler (int sig)
 	for (i = 0; i < nsd.child_count; ++i) {
 		if (nsd.children[i].pid > 0 && kill(nsd.children[i].pid, sig) == -1) {
 			log_msg(LOG_ERR, "problems killing %d: %s",
-				nsd.children[i].pid, strerror(errno));
+				(int) nsd.children[i].pid, strerror(errno));
 		}
 	}
 }
@@ -275,7 +279,7 @@ bind8_stats (struct nsd *nsd)
 		"RP", "AFSDB", "X25", "ISDN", "RT", "NSAP", "NSAP_PTR", "SIG",		/* 24 */
 		"KEY", "PX", "GPOS", "AAAA", "LOC", "NXT", "EID", "NIMLOC",		/* 32 */
 		"SRV", "ATMA", "NAPTR", "KX", "CERT", "A6", "DNAME", "SINK",		/* 40 */
-		"OPT", NULL, NULL, NULL, NULL, NULL, NULL, NULL,			/* 48 */
+		"OPT", NULL, "DS", NULL, NULL, "RRSIG", "NSEC", "DNSKEY",		/* 48 */
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,				/* 56 */
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,				/* 64 */
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,				/* 72 */
@@ -312,23 +316,23 @@ bind8_stats (struct nsd *nsd)
 	/* NSTATS */
 	t = msg = buf + snprintf(buf, MAXSYSLOGMSGLEN, "NSTATS %lu %lu",
 				 (unsigned long) now, (unsigned long) nsd->st.boot);
-	for(i = 0; i <= 255; i++) {
+	for (i = 0; i <= 255; i++) {
 		/* How much space left? */
-		if((len = buf + MAXSYSLOGMSGLEN - t) < 32) {
+		if ((len = buf + MAXSYSLOGMSGLEN - t) < 32) {
 			log_msg(LOG_INFO, "%s", buf);
 			t = msg;
 			len = buf + MAXSYSLOGMSGLEN - t;
 		}
 
-		if(nsd->st.qtype[i] != 0) {
-			if(types[i] == NULL) {
+		if (nsd->st.qtype[i] != 0) {
+			if (types[i] == NULL) {
 				t += snprintf(t, len, " TYPE%d=%lu", i, nsd->st.qtype[i]);
 			} else {
 				t += snprintf(t, len, " %s=%lu", types[i], nsd->st.qtype[i]);
 			}
 		}
 	}
-	if(t > msg)
+	if (t > msg)
 		log_msg(LOG_INFO, "%s", buf);
 
 	/* XSTATS */
@@ -369,8 +373,6 @@ main (int argc, char *argv[])
 	/* Scratch variables... */
 	int c;
 	pid_t	oldpid;
-	size_t udp_children = 1;
-	size_t tcp_children = 1;
 	size_t i;
 	struct sigaction action;
 	
@@ -392,6 +394,7 @@ main (int argc, char *argv[])
 	
 	/* Initialize the server handler... */
 	memset(&nsd, 0, sizeof(struct nsd));
+	nsd.region      = region_create(xalloc, free);
 	nsd.dbfile	= DBFILE;
 	nsd.pidfile	= PIDFILE;
 	nsd.server_kind = NSD_SERVER_MAIN;
@@ -413,6 +416,10 @@ main (int argc, char *argv[])
 	nsd.username	= USER;
 	nsd.chrootdir	= NULL;
 
+	nsd.child_count = 1;
+	nsd.maximum_tcp_count = 10;
+	nsd.current_tcp_count = 0;
+	
 	/* EDNS0 */
 	nsd.edns.max_msglen = EDNS_MAX_MESSAGE_LEN;
 	nsd.edns.opt_ok[1] = (TYPE_OPT & 0xff00) >> 8;	/* type_hi */
@@ -427,7 +434,7 @@ main (int argc, char *argv[])
 	nsd.edns.opt_err[5] = 1;			/* XXX Extended RCODE=BAD VERS */
 
 	/* Set up our default identity to gethostname(2) */
-	if(gethostname(hostname, MAXHOSTNAMELEN) == 0) {
+	if (gethostname(hostname, MAXHOSTNAMELEN) == 0) {
 		nsd.identity = hostname;
 	} else {
 		log_msg(LOG_ERR,
@@ -437,7 +444,7 @@ main (int argc, char *argv[])
 
 
 	/* Parse the command line... */
-	while((c = getopt(argc, argv, "46a:df:hi:l:N:n:p:s:u:t:X:v")) != -1) {
+	while ((c = getopt(argc, argv, "46a:df:hi:l:N:n:p:s:u:t:X:vF:L:")) != -1) {
 		switch (c) {
 		case '4':
 			for (i = 0; i < MAX_INTERFACES; ++i) {
@@ -475,17 +482,17 @@ main (int argc, char *argv[])
 		case 'N':
 			i = atoi(optarg);
 			if (i <= 0) {
-				error("number of UDP servers must be greather than zero");
+				error("number of child servers must be greather than zero");
 			} else {
-				udp_children = i;
+				nsd.child_count = i;
 			}
 			break;
 		case 'n':
 			i = atoi(optarg);
 			if (i <= 0) {
-				error("number of TCP servers must be greather than zero");
+				error("number of concurrent TCP connections must greater than zero");
 			} else {
-				tcp_children = i;
+				nsd.maximum_tcp_count = i;
 			}
 			break;
 		case 'p':
@@ -524,6 +531,14 @@ main (int argc, char *argv[])
 		case 'v':
 			version();
 			break;
+#ifndef NDEBUG
+		case 'F':
+			sscanf(optarg, "%x", &nsd_debug_facilities);
+			break;
+		case 'L':
+			sscanf(optarg, "%d", &nsd_debug_level);
+			break;
+#endif /* NDEBUG */
 		case '?':
 		default:
 			usage();
@@ -532,7 +547,7 @@ main (int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
-	if(argc != 0)
+	if (argc != 0)
 		usage();
 
 	if (strlen(nsd.identity) > UCHAR_MAX) {
@@ -541,17 +556,14 @@ main (int argc, char *argv[])
 	}
 	
 	/* Number of child servers to fork.  */
-	nsd.child_count = udp_children + tcp_children;
-	nsd.children = xalloc(nsd.child_count * sizeof(struct nsd_child));
-	for (i = 0; i < udp_children; ++i) {
-		nsd.children[i].kind = NSD_SERVER_UDP;
-	}
-	for (; i < nsd.child_count; ++i) {
-		nsd.children[i].kind = NSD_SERVER_TCP;
+	nsd.children = region_alloc(
+		nsd.region, nsd.child_count * sizeof(struct nsd_child));
+	for (i = 0; i < nsd.child_count; ++i) {
+		nsd.children[i].kind = NSD_SERVER_BOTH;
 	}
 	
 	/* We need at least one active interface */
-	if(nsd.ifs == 0) {
+	if (nsd.ifs == 0) {
 		nsd.ifs = 1;
 
 		/*
@@ -580,39 +592,39 @@ main (int argc, char *argv[])
 	}
 
 	/* Set up the address info structures with real interface/port data */
-	for (i = 0; i < nsd.ifs; ++i)
-	{
+	for (i = 0; i < nsd.ifs; ++i) {
 		/* We don't perform name-lookups */
 		if (nodes[i] != NULL)
 			hints[i].ai_flags |= AI_NUMERICHOST;
 		
 		hints[i].ai_socktype = SOCK_DGRAM;
-		if ( getaddrinfo(nodes[i], udp_port, &hints[i], &nsd.udp[i].addr) != 0)
-			usage();
+		if (getaddrinfo(nodes[i], udp_port, &hints[i], &nsd.udp[i].addr) != 0) {
+			error("cannot parse address '%s'", nodes[i]);
+		}
 		
 		hints[i].ai_socktype = SOCK_STREAM;
-		if ( getaddrinfo(nodes[i], tcp_port, &hints[i], &nsd.tcp[i].addr) != 0)
-			usage();
-
+		if (getaddrinfo(nodes[i], tcp_port, &hints[i], &nsd.tcp[i].addr) != 0) {
+			error("cannot parse address '%s'", nodes[i]);
+		}
 	}
 
 	/* Parse the username into uid and gid */
 	nsd.gid = getgid();
 	nsd.uid = getuid();
-	if(*nsd.username) {
+	if (*nsd.username) {
 		struct passwd *pwd;
-		if(isdigit(*nsd.username)) {
+		if (isdigit(*nsd.username)) {
 			char *t;
 			nsd.uid = strtol(nsd.username, &t, 10);
-			if(*t != 0) {
-				if(*t != '.' || !isdigit(*++t)) {
+			if (*t != 0) {
+				if (*t != '.' || !isdigit(*++t)) {
 					error("-u user or -u uid or -u uid.gid");
 				}
 				nsd.gid = strtol(t, &t, 10);
 			} else {
 				/* Lookup the group id in /etc/passwd */
-				if((pwd = getpwuid(nsd.uid)) == NULL) {
-					error("user id %d does not exist.", nsd.uid);
+				if ((pwd = getpwuid(nsd.uid)) == NULL) {
+					error("user id %u does not exist.", (unsigned) nsd.uid);
 				} else {
 					nsd.gid = pwd->pw_gid;
 				}
@@ -620,7 +632,7 @@ main (int argc, char *argv[])
 			}
 		} else {
 			/* Lookup the user id in /etc/passwd */
-			if((pwd = getpwnam(nsd.username)) == NULL) {
+			if ((pwd = getpwnam(nsd.username)) == NULL) {
 				error("user '%s' does not exist.", nsd.username);
 			} else {
 				nsd.uid = pwd->pw_uid;
@@ -637,14 +649,14 @@ main (int argc, char *argv[])
 	}
 	
 	/* Relativize the pathnames for chroot... */
-	if(nsd.chrootdir) {
+	if (nsd.chrootdir) {
 		int l = strlen(nsd.chrootdir);
 
-		if(strncmp(nsd.chrootdir, nsd.pidfile, l) != 0) {
+		if (strncmp(nsd.chrootdir, nsd.pidfile, l) != 0) {
 			log_msg(LOG_ERR, "%s is not relative to %s: will not chroot",
 				nsd.pidfile, nsd.chrootdir);
 			nsd.chrootdir = NULL;
-		} else if(strncmp(nsd.chrootdir, nsd.dbfile, l) != 0) {
+		} else if (strncmp(nsd.chrootdir, nsd.dbfile, l) != 0) {
 			log_msg(LOG_ERR, "%s is not relative to %s: will not chroot",
 				nsd.dbfile, nsd.chrootdir);
 			nsd.chrootdir = NULL;
@@ -652,17 +664,18 @@ main (int argc, char *argv[])
 	}
 
 	/* Do we have a running nsd? */
-	if((oldpid = readpid(nsd.pidfile)) == -1) {
-		if(errno != ENOENT) {
+	if ((oldpid = readpid(nsd.pidfile)) == -1) {
+		if (errno != ENOENT) {
 			log_msg(LOG_ERR, "can't read pidfile %s: %s",
 				nsd.pidfile, strerror(errno));
 		}
 	} else {
-		if(kill(oldpid, 0) == 0 || errno == EPERM) {
+		if (kill(oldpid, 0) == 0 || errno == EPERM) {
 			log_msg(LOG_ERR,
 				"nsd is already running as %u, stopping",
 				(unsigned) oldpid);
-			exit(0);
+			/* XXX: Stop or continue to bind port anyway? */
+/* 			exit(0); */
 		} else {
 			log_msg(LOG_ERR,
 				"...stale pid file from process %u",
@@ -689,12 +702,12 @@ main (int argc, char *argv[])
 		}
 
 		/* Detach ourselves... */
-		if(setsid() == -1) {
+		if (setsid() == -1) {
 			log_msg(LOG_ERR, "setsid() failed: %s", strerror(errno));
 			exit(1);
 		}
 
-		if((fd = open("/dev/null", O_RDWR, 0)) != -1) {
+		if ((fd = open("/dev/null", O_RDWR, 0)) != -1) {
 			(void)dup2(fd, STDIN_FILENO);
 			(void)dup2(fd, STDOUT_FILENO);
 			(void)dup2(fd, STDERR_FILENO);
@@ -721,7 +734,7 @@ main (int argc, char *argv[])
 	nsd.pid = getpid();
 
 	/* Overwrite pid... */
-	if(writepid(&nsd) == -1) {
+	if (writepid(&nsd) == -1) {
 		log_msg(LOG_ERR, "cannot overwrite the pidfile %s: %s",
 			nsd.pidfile, strerror(errno));
 	}
@@ -730,7 +743,7 @@ main (int argc, char *argv[])
 	nsd.mode = NSD_RUN;
 
 	/* Run the server... */
-	if(server_init(&nsd) != 0) {
+	if (server_init(&nsd) != 0) {
 		unlink(nsd.pidfile);
 		exit(1);
 	}
@@ -745,7 +758,7 @@ main (int argc, char *argv[])
 			*eq = '\0';
 			arg = eq + 1;
 		}
-		if (!plugin_load(plugins[i], arg)) {
+		if (!plugin_load(&nsd, plugins[i], arg)) {
 			plugin_finalize_all();
 			unlink(nsd.pidfile);
 			exit(1);
@@ -754,7 +767,7 @@ main (int argc, char *argv[])
 	free(plugins);
 #endif /* PLUGINS */
 	
-	log_msg(LOG_NOTICE, "nsd started, pid %d", nsd.pid);
+	log_msg(LOG_NOTICE, "nsd started, pid %d", (int) nsd.pid);
 
 	if (nsd.server_kind == NSD_SERVER_MAIN) {
 		server_main(&nsd);
