@@ -3,7 +3,7 @@
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2004, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -39,6 +39,8 @@
 #ifndef	_NSD_H_
 #define	_NSD_H_
 
+#include "region-allocator.h"
+
 #define	NSD_RUN	0
 #define	NSD_RELOAD 1
 #define	NSD_SHUTDOWN 2
@@ -85,12 +87,20 @@ struct nsd_socket
 
 struct nsd_child
 {
+	 /* The type of child process (UDP or TCP handler). */
 	int   kind;
+
+	/* The child's process id.  */
 	pid_t pid;
 };
 
 /* NSD configuration and run-time variables */
 struct	nsd {
+	/*
+	 * Global region that is not deallocated until NSD shuts down.
+	 */
+	region_type    *region;
+	
 	/* Run-time variables */
 	pid_t		pid;
 	int		mode;
@@ -111,6 +121,7 @@ struct	nsd {
 	const char	*chrootdir;
 	const char	*version;
 	const char	*identity;
+
 	size_t	ifs;
 
 	/* TCP specific configuration */
@@ -125,6 +136,9 @@ struct	nsd {
 		char		opt_err[OPT_LEN];
 	} edns;
 
+	int maximum_tcp_count;
+	int current_tcp_count;
+	
 #ifdef	BIND8_STATS
 
 	char	*named8_stats;
