@@ -3,7 +3,7 @@
  *
  * Erik Rozendaal, <erik@nlnetlabs.nl>
  *
- * Copyright (c) 2003, NLnet Labs. All rights reserved.
+ * Copyright (c) 2003-2004, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -43,13 +43,18 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
-
 #ifdef HAVE_SYSLOG_H
 #include <syslog.h>
 #endif /* HAVE_SYSLOG_H */
+#include <unistd.h>
 
 #include "util.h"
 
+#ifndef NDEBUG
+unsigned nsd_debug_facilities = 0xffff;
+int nsd_debug_level = 0;
+#endif
+
 static const char *global_ident = NULL;
 static log_function_type *current_log_function = log_file;
 static FILE *current_log_file = NULL;
@@ -153,6 +158,14 @@ xalloc(size_t size)
 	return result;
 }
 
+void *
+xalloc_zero(size_t size)
+{
+	void *result = xalloc(size);
+	memset(result, 0, size);
+	return result;
+}
+
 void *
 xrealloc(void *ptr, size_t size)
 {
@@ -163,3 +176,75 @@ xrealloc(void *ptr, size_t size)
 	}
 	return ptr;
 }
+
+int
+write_data(FILE *file, const void *data, size_t size)
+{
+	size_t result;
+
+	if (size == 0)
+		return 1;
+	
+	result = fwrite(data, 1, size, file);
+
+	if (result == 0) {
+		log_msg(LOG_ERR, "write failed: %s", strerror(errno));
+		return 0;
+	} else if (result < size) {
+		log_msg(LOG_ERR, "short write (disk full?)");
+		return 0;
+	} else {
+		return 1;
+	}
+}
+
+int
+timespec_compare(const struct timespec *left,
+		 const struct timespec *right)
+{
+	/* Compare seconds.  */
+	if (left->tv_sec < right->tv_sec) {
+		return -1;
+	} else if (left->tv_sec > right->tv_sec) {
+		return 1;
+	} else {
+		/* Seconds are equal, compare nanoseconds.  */
+		if (left->tv_nsec < right->tv_nsec) {
+			return -1;
+		} else if (left->tv_nsec > right->tv_nsec) {
+			return 1;
+		} else {
+			return 0;
+		}
+	}
+}
+
+
+/* One second is 1e9 nanoseconds.  */
+#define NANOSECONDS_PER_SECOND   1000000000L
+
+void
+timespec_add(struct timespec *left,
+	     const struct timespec *right)
+{
+	left->tv_sec += right->tv_sec;
+	left->tv_nsec += right->tv_nsec;
+	if (left->tv_nsec >= NANOSECONDS_PER_SECOND) {
+		/* Carry.  */
+		++left->tv_sec;
+		left->tv_nsec -= NANOSECONDS_PER_SECOND;
+	}
+}
+
+void
+timespec_subtract(struct timespec *left,
+		  const struct timespec *right)
+{
+	left->tv_sec -= right->tv_sec;
+	left->tv_nsec -= right->tv_nsec;
+	if (left->tv_nsec < 0L) {
+		/* Borrow.  */
+		--left->tv_sec;
+		left->tv_nsec += NANOSECONDS_PER_SECOND;
+	}
+}
