@@ -3,7 +3,7 @@
  *
  * Erik Rozendaal, <erik@nlnetlabs.nl>
  *
- * Copyright (c) 2003, NLnet Labs. All rights reserved.
+ * Copyright (c) 2003-2004, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -41,8 +41,16 @@
 
 #include <config.h>
 
+#include <sys/time.h>
 #include <stdarg.h>
 #include <syslog.h>
+#include <time.h>
+
+#define ALIGN_UP(n, alignment)  \
+	(((n) + (alignment) - 1) & (~((alignment) - 1)))
+#define PADDING(n, alignment)   \
+	(ALIGN_UP((n), (alignment)) - (n))
+
 
 /*
  * Initialize the logging system.  All messages are logged to stderr
@@ -99,6 +107,84 @@ void log_vmsg(int priority, const char *format, va_list args);
  * returns NULL.
  */
 void *xalloc(size_t size);
+void *xalloc_zero(size_t size);
 void *xrealloc(void *ptr, size_t size);
 
+/*
+ * Write SIZE bytes of DATA to FILE.  Report an error on failure.
+ *
+ * Returns 0 on failure, 1 on success.
+ */
+int write_data(FILE * file, const void *data, size_t size);
+
+
+/*
+ * Copy data allowing for unaligned accesses in network byte order
+ * (big endian).
+ */
+static inline void
+copy_uint16(void *dst, uint16_t data)
+{
+#ifdef ALLOW_UNALIGNED_ACCESSES
+	* (uint16_t *) dst = htons(data);
+#else
+	uint8_t *p = dst;
+	p[0] = (uint8_t) ((data >> 8) & 0xff);
+	p[1] = (uint8_t) (data & 0xff);
+#endif
+}
+
+static inline void
+copy_uint32(void *dst, uint32_t data)
+{
+#ifdef ALLOW_UNALIGNED_ACCESSES
+	* (uint32_t *) dst = htonl(data);
+#else
+	uint8_t *p = dst;
+	p[0] = (uint8_t) ((data >> 24) & 0xff);
+	p[1] = (uint8_t) ((data >> 16) & 0xff);
+	p[2] = (uint8_t) ((data >> 8) & 0xff);
+	p[3] = (uint8_t) (data & 0xff);
+#endif
+}
+
+/*
+ * Print debugging information using fprintf(3).
+ */
+#define DEBUG_PARSER           0x0001U
+#define DEBUG_ZONEC            0x0002U
+#define DEBUG_QUERY            0x0004U
+#define DEBUG_DBACCESS         0x0008U
+#define DEBUG_NAME_COMPRESSION 0x0010U
+
+#ifdef NDEBUG
+#define DEBUG(facility, level, args)  /* empty */
+#else
+extern unsigned nsd_debug_facilities;
+extern int nsd_debug_level;
+#define DEBUG(facility, level, args)				\
+	do {							\
+		if ((facility) & nsd_debug_facilities &&	\
+		    (level) <= nsd_debug_level) {		\
+			fprintf args ;				\
+		}						\
+	} while (0)
+#endif
+
+
+/*
+ * Timespec functions.
+ */
+int timespec_compare(const struct timespec *left, const struct timespec *right);
+void timespec_add(struct timespec *left, const struct timespec *right);
+void timespec_subtract(struct timespec *left, const struct timespec *right);
+
+static inline void
+timeval_to_timespec(struct timespec *left,
+		    const struct timeval *right)
+{
+	left->tv_sec = right->tv_sec;
+	left->tv_nsec = 1000 * right->tv_usec;
+}
+
 #endif /* _UTIL_H_ */
